/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of libcftp - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License (see COPYING.LGPL 
 * for details).
 *
 * $Id: utils.c,v 1.3 2000/04/22 21:14:42 malekith Exp $
 */

#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#ifdef WINDA
#include <winsock.h>
#else /* !WINDA */
#include <netinet/in.h>
#endif /* WINDA */

void *xmalloc(size_t size)
{
	void *r;
	r = malloc(size);
	if (r == 0) {
		fprintf(stderr, "xmalloc(%d): out of memory !\n", size);
		abort();
	}
	return r;
}

void *xmalloc0(size_t size)
{
	void *r;
	r = malloc(size);
	if (r == 0) {
		fprintf(stderr, "xmalloc0(%d): out of memory !\n", size);
		abort();
	}
	memset(r, 0, size);
	return r;
}

void xfree(void *p)
{
	free(p);
}

void *xrealloc(void *p, size_t ns)
{
	void *r;
	r = realloc(p, ns);
	if (r == 0) {
		fprintf(stderr, "xrealloc(%d): out of memory !\n", ns);
		abort();
	}
	return r;
}

void xfree0(void *p, size_t size)
{
	if (p)
		memset(p, 0, size);
	free(p);
}

char *xstrdup(const char *str)
{
	char *r;
	r = (char*)malloc(strlen(str) + 1);
	if (r == 0) {
		fprintf(stderr, "xstrdup(): out of memory !\n");
		abort();
	}
	strcpy(r, str);
	return r;
}

#if defined(strcasecmp)
int xstrcasecmp(const char *left, const char *right)
{
	return (strcasecmp(left, right));
}
#else 
int xstrcasecmp(const char *left, const char *right)
{
	while (*left && *right && tolower(*left) == tolower(*right))
		left++, right++;
	if (tolower(*left) == tolower(*right))
		return 0;
	else if (tolower(*left) < tolower(*right))
		return -1;
	else
		return 1;
}
#endif

int h2n(int d)
{
	return htonl(d);
}

int n2h(int d)
{
	return ntohl(d);
}

#ifdef WINDA
void sleep(int sec)
{
	Sleep(sec * 1000);
}

void usleep(int msec)
{
	Sleep(msec/1000);
}

#endif

