#include "config.h"
#ifdef WINDA
#include <winsock.h>
struct wsa_error {
int error;
char *desc;
};
static struct wsa_error  errors[] = {
{ WSAEINTR, "A blocking operation was interrupted by a call to WSACancelBlockingCall." },
{ WSAEBADF, "The file handle supplied is not valid." },
{ WSAEACCES, "An attempt was made to access a socket in a way forbidden by its access permissions." },
{ WSAEFAULT, "The system detected an invalid pointer address in attempting to use a pointer argument in a call." },
{ WSAEINVAL, "An invalid argument was supplied." },
{ WSAEMFILE, "Too many open sockets." },
{ WSAEWOULDBLOCK, "A non-blocking socket operation could not be completed immediately." },
{ WSAEINPROGRESS, "A blocking operation is currently executing." },
{ WSAEALREADY, "An operation was attempted on a non-blocking socket that already had an operation in progress." },
{ WSAENOTSOCK, "An operation was attempted on something that is not a socket." },
{ WSAEDESTADDRREQ, "A required address was omitted from an operation on a socket." },
{ WSAEMSGSIZE, "A message sent on a datagram socket was larger than the internal message buffer or some other network limit, or the buffer used to receive a datagram into was smaller than the datagram itself." },
{ WSAEPROTOTYPE, "A protocol was specified in the socket function call that does not support the semantics of the socket type requested." },
{ WSAENOPROTOOPT, "An unknown, invalid, or unsupported option or level was specified in a getsockopt or setsockopt call." },
{ WSAEPROTONOSUPPORT, "The requested protocol has not been configured into the system, or no implementation for it exists." },
{ WSAESOCKTNOSUPPORT, "The support for the specified socket type does not exist in this address family." },
{ WSAEOPNOTSUPP, "The attempted operation is not supported for the type of object referenced." },
{ WSAEPFNOSUPPORT, "The protocol family has not been configured into the system or no implementation for it exists." },
{ WSAEAFNOSUPPORT, "An address incompatible with the requested protocol was used." },
{ WSAEADDRINUSE, "Only one usage of each socket address (protocol/network address/port)" },
{ WSAEADDRNOTAVAIL, "The requested address is not valid in its context." },
{ WSAENETDOWN, "A socket operation encountered a dead network." },
{ WSAENETUNREACH, "A socket operation was attempted to an unreachable network." },
{ WSAENETRESET, "The connection has been broken due to keep-alive activity detecting a failure while the operation was in progress." },
{ WSAECONNABORTED, "An established connection was aborted by the software in your host machine." },
{ WSAECONNRESET, "An existing connection was forcibly closed by the remote host." },
{ WSAENOBUFS, "An operation on a socket could not be performed because the system lacked sufficient buffer space or because a queue was full." },
{ WSAEISCONN, "A connect request was made on an already connected socket." },
{ WSAENOTCONN, "A request to send or receive data was disallowed because the socket is not connected and (when sending on a datagram socket using a sendto call) no address was supplied." },
{ WSAESHUTDOWN, "A request to send or receive data was disallowed because the socket had already been shut down in that direction with a previous shutdown call." },
{ WSAETOOMANYREFS, "Too many references to some kernel object." },
{ WSAETIMEDOUT, "A connection attempt failed because the connected party did not properly respond after a period of time, or established connection failed because connected host has failed to respond." },
{ WSAECONNREFUSED, "No connection could be made because the target machine actively refused it." },
{ WSAELOOP, "Cannot translate name." },
{ WSAENAMETOOLONG, "Name component or name was too long." },
{ WSAEHOSTDOWN, "A socket operation failed because the destination host was down." },
{ WSAEHOSTUNREACH, "A socket operation was attempted to an unreachable host." },
{ WSAENOTEMPTY, "Cannot remove a directory that is not empty." },
{ WSAEPROCLIM, "A Windows Sockets implementation may have a limit on the number of applications that may use it simultaneously." },
{ WSAEUSERS, "Ran out of quota." },
{ WSAEDQUOT, "Ran out of disk quota." },
{ WSAESTALE, "File handle reference is no longer available." },
{ WSAEREMOTE, "Item is not available locally." },
{ WSASYSNOTREADY, "WSAStartup cannot function at this time because the underlying system it uses to provide network services is currently unavailable." },
{ WSAVERNOTSUPPORTED, "The Windows Sockets version requested is not supported." },
{ WSANOTINITIALISED, "Either the application has not called WSAStartup, or WSAStartup failed." },
{ WSAEDISCON, "Returned by WSARecv or WSARecvFrom to indicate the remote party has initiated a graceful shutdown sequence." },
{ WSAENOMORE, "No more results can be returned by WSALookupServiceNext." },
{ WSAECANCELLED, "A call to WSALookupServiceEnd was made while this call was still processing. The call has been canceled." },
{ WSAEINVALIDPROCTABLE, "The procedure call table is invalid." },
{ WSAEINVALIDPROVIDER, "The requested service provider is invalid." },
{ WSAEPROVIDERFAILEDINIT, "The requested service provider could not be loaded or initialized." },
{ WSASYSCALLFAILURE, "A system call that should never fail has failed." },
//{ WSASERVICE, "No such service is known. The service cannot be found in the specified name space." },
//{ WSATYPE, "The specified class was not found." },
{ WSAEREFUSED, "A database query failed because it was actively refused." },
//{ WSAHOST, "No such host is known." },
//{ WSATRY, "This is usually a temporary error during hostname resolution and means that the local server did not receive a response from an authoritative server." },
//{ WSANO, "A non-recoverable error occurred during a database lookup." },
//{ WSANO, "The requested name is valid and was found in the database, but it does not have the correct associated data being resolved for." },
{ 0, 0 }
};
char *get_wsa_error()
{
int e;
struct wsa_error *p;

e = WSAGetLastError();
if(e==0)return "No error.";
for(p = errors;p->error;p++)
if(p->error==e)return p->desc;
return "Unknown error.";
}

void init_wsa()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	wVersionRequested = MAKEWORD( 2, 0 );

	WSAStartup( wVersionRequested, &wsaData );
}

void kill_wsa()
{
	WSACleanup();
}

#endif
