#include "config.h"
#include "server.h"

#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#ifdef IPv6
int use_ipv6 = 0;
#endif

static char *userid(struct sockaddr_in *sin_local, struct sockaddr_in *sin,
		    int size)
{
	int sock;
	char buf[200];
	int r;
	char *p;
	int lport, rport;

#ifdef IPv6
	sock = socket(use_ipv6 ? AF_INET6 : AF_INET, SOCK_STREAM, 0);
#else
	sock = socket(AF_INET, SOCK_STREAM, 0);
#endif

	lport = ntohs(sin_local->sin_port);
	rport = ntohs(sin->sin_port);

	if (sock == -1)
		return xstrdup("can't-get-socket");

	sin_local->sin_port = htons(0);
	/* bind to the same interface */
	bind(sock, (struct sockaddr *) sin_local, size);

	sin->sin_port = htons(113);
	if (connect(sock, (struct sockaddr *) sin, size) == -1) {
		einfo("connect");
		close(sock);
		return xstrdup("no-ident");
	}

	sprintf(buf, "%d , %d\n", rport, lport);

	write(sock, buf, strlen(buf));
	r = read(sock, buf, sizeof(buf) - 1);
	close(sock);

	if (r == -1)
		return xstrdup("ident-error");

	buf[r] = 0;
	p = buf;
	
	debug("ident resp: '%s'", p);
	
	/* skip ports */
	while (*p && *p != ':')
		p++;
	if (*p == ':')
		p++;
	while (isspace(*p))
		p++;
		
	if (strncmp(p, "USERID", 6) != 0)
		return xstrdup("bad-ident-response1");

	/* skip sys-id -- this is probably UNIX anyway */
	while (*p && *p != ':')
		p++;
	if (*p == ':')
		p++;
	while (*p && *p != ':')
		p++;
	while (isspace(*p))
		p++;

	if (*p != ':')
		return xstrdup("bad-ident-response2");
	p++;
	if (strchr(p, '\r'))
		*strchr(p, '\r') = 0;
	if (strchr(p, '\n'))
		*strchr(p, '\n') = 0;
	return xstrdup(p);
}

void ident_user(int sock)
{
	struct sockaddr_in sin, sin_local;
#ifdef IPv6
	struct sockaddr_in6 sin6, sin6_local;
#endif
	socklen_t len;
	struct hostent *h = 0;
	char *hname = "unknown", *uname;

	len = sizeof(sin_local);
	if (getsockname(sock, (struct sockaddr *) &sin_local, &len) < 0) {
		warn("cannot get sockaddr");
		return;
	}
	len = sizeof(sin);
	if (getpeername(sock, (struct sockaddr *) &sin, &len) < 0) {
		warn("cannot get peer sockaddr");
		return;
	}
#ifdef IPv6
	len = sizeof(sin6_local);
	if (getsockname(sock, (struct sockaddr *) &sin6_local, &len) < 0) {
		warn("cannot get sockaddr");
		return;
	}
	if (len == sizeof(sin6_local)) {
		len = sizeof(sin6);
		if (getpeername(sock, (struct sockaddr *) &sin6, &len) < 0) {
			warn("cannot get peer sockaddr");
			return;
		}
		use_ipv6 = 1;
	}
#endif

	if (!conf_get_bool("DontAskDNS", 0)) {
#ifdef IPv6
		if (use_ipv6)
			h =
			    gethostbyaddr(sin6.sin6_addr.s6_addr, 16,
					  AF_INET6);
		else
#endif
			h = gethostbyaddr((char *) &sin.sin_addr,
					  sizeof(sin.sin_addr), AF_INET);
	}

	if (h)
		hname = (char*)h->h_name;
	hname = xstrdup(hname);

	if (!conf_get_bool("DontAskIdent", 0)) {
#ifdef IPv6
		if (use_ipv6)
			uname =
			    userid((struct sockaddr_in *) &sin6_local,
				   (struct sockaddr_in *) &sin6, 
				   sizeof(sin6));
		else
#endif
			uname = userid(&sin_local, &sin, sizeof(sin));
	} else
		uname = xstrdup("didn't-ask");

	{
		char ipbuf[100];
#ifdef IPv6
		if (use_ipv6)
			inet_ntop(AF_INET6, sin6.sin6_addr.s6_addr, ipbuf,
				  99);
		else
#endif
			strncpy(ipbuf, inet_ntoa(sin.sin_addr), 99);

		notice("connection from %s@%s[%s]", uname, hname, ipbuf);
	}

	/* maybe use for future logs ? */
	xfree(hname);
	xfree(uname);
}
