/*
 * Copyright (c) y2k Micha Moskal <malekith@topnet.pl>
 *
 * This file is part of cryptoFTP - cftp protocol implementation.
 * CryptoFTP is free software. You can redistribute it and/or modify it under
 * the terms of the GNU General Public License (see COPYING.GPL for details).
 *
 * $Id: standalone.c,v 1.6 2000/05/05 22:33:10 malekith Exp $
 */

#include "config.h"

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>

#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <sys/wait.h>
#include <ctype.h>

#ifdef IPv6
int make_socket6(int port)
{
	int sock;
	struct sockaddr_in6 name;

	/* Create the socket. */
	sock = socket(PF_INET6, SOCK_STREAM, 0);
	if (sock == -1) {
		perror("socket6");
		printf("falling back to ipv4\n");
		return -1;
	}

	/* Give the socket a name. */
	name.sin6_family = AF_INET6;
	name.sin6_port = htons(port);
	name.sin6_addr = in6addr_any;
	if (bind(sock, (struct sockaddr *) &name, sizeof(name)) < 0) {
		perror("bind6");
		printf("falling back to ipv4\n");
		close(sock);
		return -1;
	}

	return sock;
}
#endif

int make_socket(int port)
{
	int sock;
	struct sockaddr_in name;

#ifdef IPv6
	sock = make_socket6(port);
	if (sock != -1)
		return sock;
#endif
		
	/* Create the socket. */
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if (sock == -1) {
		perror("socketX");
		exit(1);
	}

	/* Give the socket a name. */
	name.sin_family = AF_INET;
	name.sin_port = htons(port);
	name.sin_addr.s_addr = htonl(INADDR_ANY);
	if (bind(sock, (struct sockaddr *) &name, sizeof(name)) < 0) {
		perror("bind");
		exit(1);
	}

	return sock;
}

int inetd_main(int,char**,int);

void child(int s)
{
	wait(0);
	signal(s, child);
}

void byebye()
{
	close(0);
	close(1);
	close(2);
}

int main(int argc, char **argv)
{
	int sock;
        int port = 26;

        if (argc > 1 && isdigit(*argv[1])) {
                port = atoi(argv[1]);
        	argv[1] = argv[0];
                argv++; argc--;
        }

	/* Create the socket and set it up to accept connections. */
	sock = make_socket(port);
	if (listen(sock, 16) < 0) {
		perror("listen");
		exit(1);
	}

	signal(SIGCHLD, child);

	while (1) {
		int newfd;
#ifndef NO_FORK
		int pid;
#endif

		newfd = accept(sock, 0, 0);
		if (newfd == -1) {
			perror("accept");
			exit(1);
		}
		printf("client accepted\n");

#ifndef NO_FORK
		pid = fork();
		if (pid == -1) {
			perror("fork");
			exit(1);
		}
		
		if (pid == 0) {
			close(sock);
			return inetd_main(argc, argv, newfd);
		}
#else
		inetd_main(argc, argv, newfd);
#endif
	}
}
